@@ -6,7 +6,9 @@ package Excel::Writer::XLSX::Package::Metadata;
 #
 # Used in conjunction with Excel::Writer::XLSX
 #
-# Copyright 2000-2023, John McNamara, jmcnamara@cpan.org
+# Copyright 2000-2024, John McNamara, jmcnamara@cpan.org
+#
+# SPDX-License-Identifier: Artistic-1.0-Perl OR GPL-1.0-or-later
 #
 # Documentation after __END__
 #
@@ -20,7 +22,7 @@ use Carp;
 use Excel::Writer::XLSX::Package::XMLwriter;
 
 our @ISA     = qw(Excel::Writer::XLSX::Package::XMLwriter);
-our $VERSION = '1.10';
+our $VERSION = '1.12';
 
 
 ###############################################################################
@@ -42,6 +44,10 @@ sub new {
     my $fh    = shift;
     my $self  = Excel::Writer::XLSX::Package::XMLwriter->new( $fh );
 
+    $self->{_has_dynamic_functions} = 0;
+    $self->{_has_embedded_images}   = 0;
+    $self->{_num_embedded_images}   = 0;
+
     bless $self, $class;
 
     return $self;
@@ -58,6 +64,10 @@ sub _assemble_xml_file {
 
     my $self = shift;
 
+    if ( $self->{_num_embedded_images} > 0 ) {
+        $self->{_has_embedded_images} = 1;
+    }
+
     $self->xml_declaration;
 
     # Write the metadata element.
@@ -67,10 +77,12 @@ sub _assemble_xml_file {
     $self->_write_metadata_types();
 
     # Write the futureMetadata element.
-    $self->_write_future_metadata();
+    $self->_write_cell_future_metadata()  if $self->{_has_dynamic_functions};
+    $self->_write_value_future_metadata() if $self->{_has_embedded_images};
 
-    # Write the cellMetadata element.
-    $self->_write_cell_metadata();
+    # Write the valueMetadata element.
+    $self->_write_cell_metadata()  if $self->{_has_dynamic_functions};
+    $self->_write_value_metadata() if $self->{_has_embedded_images};
 
     $self->xml_end_tag( 'metadata' );
 
@@ -94,16 +106,30 @@ sub _assemble_xml_file {
 #
 sub _write_metadata {
 
-    my $self  = shift;
-    my $xmlns = 'http://schemas.openxmlformats.org/spreadsheetml/2006/main';
-    my $xmlns_xda =
-      'http://schemas.microsoft.com/office/spreadsheetml/2017/dynamicarray';
+    my $self = shift;
 
     my @attributes = (
-        'xmlns'     => $xmlns,
-        'xmlns:xda' => $xmlns_xda,
+        'xmlns' =>
+        'http://schemas.openxmlformats.org/spreadsheetml/2006/main'
     );
 
+
+    if ( $self->{_has_embedded_images} ) {
+        push @attributes,
+          (
+              'xmlns:xlrd' =>
+              'http://schemas.microsoft.com/office/spreadsheetml/2017/richdata'
+          );
+    }
+
+    if ( $self->{_has_dynamic_functions} ) {
+        push @attributes,
+          (
+              'xmlns:xda' =>
+              'http://schemas.microsoft.com/office/spreadsheetml/2017/dynamicarray'
+          );
+    }
+
     $self->xml_start_tag( 'metadata', @attributes );
 }
 
@@ -117,13 +143,15 @@ sub _write_metadata {
 sub _write_metadata_types {
 
     my $self  = shift;
+    my $count = $self->{_has_dynamic_functions} + $self->{_has_embedded_images};
 
-    my @attributes = ( 'count' => 1 );
+    my @attributes = ( 'count' => $count );
 
     $self->xml_start_tag( 'metadataTypes', @attributes );
 
     # Write the metadataType element.
-    $self->_write_metadata_type();
+    $self->_write_cell_metadata_type()  if $self->{_has_dynamic_functions};
+    $self->_write_value_metadata_type() if $self->{_has_embedded_images};
 
     $self->xml_end_tag( 'metadataTypes' );
 }
@@ -131,11 +159,11 @@ sub _write_metadata_types {
 
 ##############################################################################
 #
-# _write_metadata_type()
+# _write_cell_metadata_type()
 #
 # Write the <metadataType> element.
 #
-sub _write_metadata_type {
+sub _write_cell_metadata_type {
 
     my $self = shift;
 
@@ -161,11 +189,40 @@ sub _write_metadata_type {
 
 ##############################################################################
 #
-# _write_future_metadata()
+# _write_value_metadata_type()
+#
+# Write the <metadataType> element.
+#
+sub _write_value_metadata_type {
+
+    my $self = shift;
+
+    my @attributes = (
+        'name'                => 'XLRICHVALUE',
+        'minSupportedVersion' => 120000,
+        'copy'                => 1,
+        'pasteAll'            => 1,
+        'pasteValues'         => 1,
+        'merge'               => 1,
+        'splitFirst'          => 1,
+        'rowColShift'         => 1,
+        'clearFormats'        => 1,
+        'clearComments'       => 1,
+        'assign'              => 1,
+        'coerce'              => 1,
+    );
+
+    $self->xml_empty_tag( 'metadataType', @attributes );
+}
+
+
+##############################################################################
+#
+# _write_cell_future_metadata()
 #
 # Write the <futureMetadata> element.
 #
-sub _write_future_metadata {
+sub _write_cell_future_metadata {
 
     my $self  = shift;
 
@@ -179,7 +236,7 @@ sub _write_future_metadata {
     $self->xml_start_tag( 'extLst' );
 
     # Write the ext element.
-    $self->_write_ext();
+    $self->_write_cell_ext();
 
     $self->xml_end_tag( 'ext' );
     $self->xml_end_tag( 'extLst' );
@@ -191,13 +248,48 @@ sub _write_future_metadata {
 
 ##############################################################################
 #
-# _write_ext()
+# _write_value_future_metadata()
+#
+# Write the <futureMetadata> element.
+#
+sub _write_value_future_metadata {
+
+    my $self       = shift;
+    my $num_images = $self->{_num_embedded_images};
+
+    my @attributes = (
+        'name'  => 'XLRICHVALUE',
+        'count' => $num_images,
+    );
+
+    $self->xml_start_tag( 'futureMetadata', @attributes );
+
+    for my $i ( 0 .. $num_images - 1 ) {
+        $self->xml_start_tag( 'bk' );
+        $self->xml_start_tag( 'extLst' );
+
+        # Write the ext element.
+        $self->_write_value_ext( $i );
+
+        $self->xml_end_tag( 'ext' );
+        $self->xml_end_tag( 'extLst' );
+        $self->xml_end_tag( 'bk' );
+    }
+
+    $self->xml_end_tag( 'futureMetadata' );
+}
+
+
+##############################################################################
+#
+# _write_cell_ext()
 #
 # Write the <ext> element.
 #
-sub _write_ext {
+sub _write_cell_ext {
 
     my $self = shift;
+    my $uri = shift;
 
     my @attributes = ( 'uri' => '{bdbb8cdc-fa1e-496e-a857-3c3f30c029c3}' );
 
@@ -208,6 +300,25 @@ sub _write_ext {
 }
 
 
+##############################################################################
+#
+# _write_value_ext()
+#
+# Write the <ext> element.
+#
+sub _write_value_ext {
+
+    my $self = shift;
+    my $num  = shift;
+
+    my @attributes = ( 'uri' => '{3e2802c4-a4d2-4d8b-9148-e3be6c30e623}' );
+
+    $self->xml_start_tag( 'ext', @attributes );
+
+    # Write the <xlrd:rvb> element.
+    $self->_write_xlrd_rvb( $num );
+}
+
 ##############################################################################
 #
 # _write_xda_dynamic_array_properties()
@@ -244,13 +355,45 @@ sub _write_cell_metadata {
     $self->xml_start_tag( 'bk' );
 
     # Write the rc element.
-    $self->_write_rc();
+    $self->_write_rc(1, 0);
 
     $self->xml_end_tag( 'bk' );
     $self->xml_end_tag( 'cellMetadata' );
 }
 
 
+##############################################################################
+#
+# _write_value_metadata()
+#
+# Write the <valueMetadata> element.
+#
+sub _write_value_metadata {
+
+    my $self  = shift;
+    my $count = $self->{_num_embedded_images};
+    my $type  = 1;
+
+    if ($self->{_has_dynamic_functions}) {
+        $type  = 2;
+    }
+    my @attributes = ( 'count' => $count, );
+
+    $self->xml_start_tag( 'valueMetadata', @attributes );
+
+    for my $i ( 0 .. $count - 1 ) {
+        $self->xml_start_tag( 'bk' );
+
+        # Write the rc element.
+        $self->_write_rc( $type, $i );
+
+        $self->xml_end_tag( 'bk' );
+    }
+
+    $self->xml_end_tag( 'valueMetadata' );
+}
+
+
 ##############################################################################
 #
 # _write_rc()
@@ -259,17 +402,36 @@ sub _write_cell_metadata {
 #
 sub _write_rc {
 
-    my $self = shift;
+    my $self  = shift;
+    my $type  = shift;
+    my $value = shift;
 
     my @attributes = (
-        't' => 1,
-        'v' => 0,
+        't' => $type,
+        'v' => $value,
     );
 
     $self->xml_empty_tag( 'rc', @attributes );
 }
 
 
+##############################################################################
+#
+# _write_xlrd_rvb()
+#
+# Write the <xlrd:rvb> element.
+#
+sub _write_xlrd_rvb {
+
+    my $self  = shift;
+    my $value = shift;
+
+    my @attributes = ( 'i' => $value, );
+
+    $self->xml_empty_tag( 'xlrd:rvb', @attributes );
+}
+
+
 1;
 
 
@@ -295,13 +457,14 @@ John McNamara jmcnamara@cpan.org
 
 =head1 COPYRIGHT
 
-(c) MM-MMXXIII, John McNamara.
+
+(c) MM-MMXIV, John McNamara.
 
 All Rights Reserved. This module is free software. It may be used, redistributed and/or modified under the same terms as Perl itself.
 
 =head1 LICENSE
 
-Either the Perl Artistic Licence L<http://dev.perl.org/licenses/artistic.html> or the GPL L<http://www.opensource.org/licenses/gpl-license.php>.
+Either the Perl Artistic Licence L<https://dev.perl.org/licenses/artistic.html> or the GNU General Public License v1.0 or later L<https://dev.perl.org/licenses/gpl1.html>.
 
 =head1 DISCLAIMER OF WARRANTY
 
