@@ -6,7 +6,9 @@ package Excel::Writer::XLSX::Package::Styles;
 #
 # Used in conjunction with Excel::Writer::XLSX
 #
-# Copyright 2000-2021, John McNamara, jmcnamara@cpan.org
+# Copyright 2000-2024, John McNamara, jmcnamara@cpan.org
+#
+# SPDX-License-Identifier: Artistic-1.0-Perl OR GPL-1.0-or-later
 #
 # Documentation after __END__
 #
@@ -20,7 +22,7 @@ use Carp;
 use Excel::Writer::XLSX::Package::XMLwriter;
 
 our @ISA     = qw(Excel::Writer::XLSX::Package::XMLwriter);
-our $VERSION = '1.09';
+our $VERSION = '1.12';
 
 
 ###############################################################################
@@ -45,7 +47,7 @@ sub new {
     $self->{_xf_formats}         = undef;
     $self->{_palette}            = [];
     $self->{_font_count}         = 0;
-    $self->{_num_format_count}   = 0;
+    $self->{_num_formats}        = [];
     $self->{_border_count}       = 0;
     $self->{_fill_count}         = 0;
     $self->{_custom_colors}      = [];
@@ -126,7 +128,7 @@ sub _set_style_properties {
     $self->{_xf_formats}         = shift;
     $self->{_palette}            = shift;
     $self->{_font_count}         = shift;
-    $self->{_num_format_count}   = shift;
+    $self->{_num_formats}        = shift;
     $self->{_border_count}       = shift;
     $self->{_fill_count}         = shift;
     $self->{_custom_colors}      = shift;
@@ -160,6 +162,12 @@ sub _get_palette_color {
         return "FF" . uc( $1 );
     }
 
+    # Handle automatic color as a special case.
+    if ($index == 0x40) {
+        return "Automatic";
+    }
+
+
     # Adjust the colour index.
     $index -= 8;
 
@@ -203,7 +211,7 @@ sub _write_style_sheet {
 sub _write_num_fmts {
 
     my $self  = shift;
-    my $count = $self->{_num_format_count};
+    my $count = @{ $self->{_num_formats} };
 
     return unless $count;
 
@@ -212,12 +220,11 @@ sub _write_num_fmts {
     $self->xml_start_tag( 'numFmts', @attributes );
 
     # Write the numFmts elements.
-    for my $format ( @{ $self->{_xf_formats} } ) {
+    my $index = 164;
+    for my $num_format ( @{ $self->{_num_formats} } ) {
 
-        # Ignore built-in number formats, i.e., < 164.
-        next unless $format->{_num_format_index} >= 164;
-        $self->_write_num_fmt( $format->{_num_format_index},
-            $format->{_num_format} );
+        $self->_write_num_fmt( $index, $num_format );
+        $index++;
     }
 
     $self->xml_end_tag( 'numFmts' );
@@ -376,7 +383,9 @@ sub _write_font {
     elsif ( my $color = $format->{_color} ) {
         $color = $self->_get_palette_color( $color );
 
-        $self->_write_color( 'rgb' => $color );
+        if ($color ne 'Automatic') {
+            $self->_write_color( 'rgb' => $color );
+        }
     }
     elsif ( !$dxf_format ) {
         $self->_write_color( 'theme' => 1 );
@@ -615,12 +624,17 @@ sub _write_fill {
 
     if ( $fg_color ) {
         $fg_color = $self->_get_palette_color( $fg_color );
-        $self->xml_empty_tag( 'fgColor', 'rgb' => $fg_color );
+        if ($fg_color ne 'Automatic') {
+            $self->xml_empty_tag( 'fgColor', 'rgb' => $fg_color );
+        }
     }
 
     if ( $bg_color ) {
         $bg_color = $self->_get_palette_color( $bg_color );
-        $self->xml_empty_tag( 'bgColor', 'rgb' => $bg_color );
+
+        if ($bg_color ne 'Automatic') {
+            $self->xml_empty_tag( 'bgColor', 'rgb' => $bg_color );
+        }
     }
     else {
         if ( !$dxf_format && $format->{_pattern} <= 1) {
@@ -776,13 +790,13 @@ sub _write_sub_border {
 
     );
 
-
     push @attributes, ( style => $border_styles[$style] );
 
     $self->xml_start_tag( $type, @attributes );
 
-    if ( $color ) {
+    if ( $color && $color != 64 ) {
         $color = $self->_get_palette_color( $color );
+
         $self->xml_empty_tag( 'color', 'rgb' => $color );
     }
     else {
@@ -915,6 +929,10 @@ sub _write_xf {
     );
 
 
+    if ( $format->{_quote_prefix} ) {
+        push @attributes, ( 'quotePrefix' => 1 );
+    }
+
     if ( $format->{_num_format_index} > 0 ) {
         push @attributes, ( 'applyNumberFormat' => 1 );
     }
@@ -1193,13 +1211,13 @@ John McNamara jmcnamara@cpan.org
 
 =head1 COPYRIGHT
 
-(c) MM-MMXXI, John McNamara.
+(c) MM-MMXIV, John McNamara.
 
 All Rights Reserved. This module is free software. It may be used, redistributed and/or modified under the same terms as Perl itself.
 
 =head1 LICENSE
 
-Either the Perl Artistic Licence L<http://dev.perl.org/licenses/artistic.html> or the GPL L<http://www.opensource.org/licenses/gpl-license.php>.
+Either the Perl Artistic Licence L<https://dev.perl.org/licenses/artistic.html> or the GNU General Public License v1.0 or later L<https://dev.perl.org/licenses/gpl1.html>.
 
 =head1 DISCLAIMER OF WARRANTY
 
