@@ -6,7 +6,9 @@ package Excel::Writer::XLSX::Package::Table;
 #
 # Used in conjunction with Excel::Writer::XLSX
 #
-# Copyright 2000-2023, John McNamara, jmcnamara@cpan.org
+# Copyright 2000-2024, John McNamara, jmcnamara@cpan.org
+#
+# SPDX-License-Identifier: Artistic-1.0-Perl OR GPL-1.0-or-later
 #
 # Documentation after __END__
 #
@@ -20,7 +22,7 @@ use Carp;
 use Excel::Writer::XLSX::Package::XMLwriter;
 
 our @ISA     = qw(Excel::Writer::XLSX::Package::XMLwriter);
-our $VERSION = '1.11';
+our $VERSION = '1.12';
 
 
 ###############################################################################
@@ -227,11 +229,20 @@ sub _write_table_column {
         push @attributes, ( dataDxfId => $col_data->{_format} );
     }
 
-    if ( $col_data->{_formula} ) {
+    if ( $col_data->{_formula} || $col_data->{_custom_total} ) {
         $self->xml_start_tag( 'tableColumn', @attributes );
 
-        # Write the calculatedColumnFormula element.
-        $self->_write_calculated_column_formula( $col_data->{_formula} );
+
+        if ($col_data->{_formula}) {
+            # Write the calculatedColumnFormula element.
+            $self->_write_calculated_column_formula( $col_data->{_formula} );
+        }
+
+        if ($col_data->{_custom_total}) {
+            # Write the totalsRowFormula  element.
+            $self->_write_totals_row_formula( $col_data->{_custom_total} );
+        }
+
 
         $self->xml_end_tag( 'tableColumn' );
     }
@@ -288,6 +299,21 @@ sub _write_calculated_column_formula {
 }
 
 
+##############################################################################
+#
+# _write_totals_row_formula()
+#
+# Write the <totalsRowFormula> element.
+#
+sub _write_totals_row_formula {
+
+    my $self    = shift;
+    my $formula = shift;
+
+    $self->xml_data_element( 'totalsRowFormula', $formula );
+}
+
+
 1;
 
 
@@ -313,13 +339,13 @@ John McNamara jmcnamara@cpan.org
 
 =head1 COPYRIGHT
 
-(c) MM-MMXXIII, John McNamara.
+(c) MM-MMXIV, John McNamara.
 
 All Rights Reserved. This module is free software. It may be used, redistributed and/or modified under the same terms as Perl itself.
 
 =head1 LICENSE
 
-Either the Perl Artistic Licence L<http://dev.perl.org/licenses/artistic.html> or the GPL L<http://www.opensource.org/licenses/gpl-license.php>.
+Either the Perl Artistic Licence L<https://dev.perl.org/licenses/artistic.html> or the GNU General Public License v1.0 or later L<https://dev.perl.org/licenses/gpl1.html>.
 
 =head1 DISCLAIMER OF WARRANTY
 
